package com.fjhx.jxst.service.sales.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.account.entity.account.po.AccountManagement;
import com.fjhx.account.service.account.AccountManagementService;
import com.fjhx.area.utils.CustomizeAreaUtil;
import com.fjhx.common.entity.corporation.po.Corporation;
import com.fjhx.common.service.corporation.CorporationService;
import com.fjhx.common.utils.Assert;
import com.fjhx.customer.entity.customer.po.Customer;
import com.fjhx.customer.entity.customer.po.CustomerUser;
import com.fjhx.customer.service.customer.CustomerService;
import com.fjhx.customer.service.customer.CustomerUserService;
import com.fjhx.item.service.product.ProductInfoService;
import com.fjhx.jxst.entity.sales.dto.SalesContractDto;
import com.fjhx.jxst.entity.sales.dto.SalesContractSelectDto;
import com.fjhx.jxst.entity.sales.po.SalesContract;
import com.fjhx.jxst.entity.sales.po.SalesContractDetails;
import com.fjhx.jxst.entity.sales.vo.SalesContractDetailsVo;
import com.fjhx.jxst.entity.sales.vo.SalesContractVo;
import com.fjhx.jxst.mapper.sales.SalesContractMapper;
import com.fjhx.jxst.service.sales.SalesContractDetailsService;
import com.fjhx.jxst.service.sales.SalesContractService;
import com.fjhx.jxst.utils.CodeEnum;
import com.fjhx.wms.entity.stock.vo.StockVo;
import com.ruoyi.common.utils.wrapper.IWrapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * <p>
 * 销售合同 服务实现类
 * </p>
 *
 * @author
 * @since 2023-05-16
 */
@Service
public class SalesContractServiceImpl extends ServiceImpl<SalesContractMapper, SalesContract> implements SalesContractService {

    @Autowired
    private SalesContractDetailsService salesContractDetailsService;
    @Autowired
    private CustomerService customerService;
    @Autowired
    private ProductInfoService productInfoService;
    @Autowired
    private CustomerUserService customerUserService;
    @Autowired
    private AccountManagementService accountManagementService;
    @Autowired
    private CorporationService corporationService;

    @Override
    public Page<SalesContractVo> getPage(SalesContractSelectDto dto) {
        IWrapper<SalesContract> wrapper = getWrapper();
        if (ObjectUtil.isNotEmpty(dto.getKeyword())) {
            List<Customer> list = customerService.list(q -> q.like(Customer::getName, dto.getKeyword()));
            if (ObjectUtil.isNotEmpty(list)) {
                List<Long> ids = list.stream().map(Customer::getId).collect(Collectors.toList());
                wrapper.and(q -> q.in(SalesContract::getCustomerId, ids).or().like(SalesContractVo::getContractAmount, dto.getKeyword()));
            }else{
                wrapper.and(q -> q
                        .like(SalesContractVo::getContractAmount, dto.getKeyword())
                        .or().like(SalesContract::getCode,dto.getKeyword())
                );
            }
        }
        wrapper.orderByDesc("sc", SalesContract::getId);
        Page<SalesContractVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<SalesContractVo> records = page.getRecords();
        //赋值客户名称
        customerService.attributeAssign(records, SalesContractVo::getCustomerId, (item, customer) -> {
            item.setCustomerName(customer.getName());
        });
        return page;
    }

    @Override
    public SalesContractVo detail(Long id) {
        SalesContract salesContract = this.getById(id);
        Assert.notEmpty(salesContract,"查询不到合同信息");
        SalesContractVo result = BeanUtil.toBean(salesContract, SalesContractVo.class);
        List<SalesContractDetails> salesContractDetailsList = salesContractDetailsService.list(q -> q.eq(SalesContractDetails::getSalesContractId, result.getId()));
        List<SalesContractDetailsVo> salesContractDetailsVos = BeanUtil.copyToList(salesContractDetailsList, SalesContractDetailsVo.class);
        //赋值产品信息
        productInfoService.attributeAssign(salesContractDetailsVos, SalesContractDetailsVo::getProductId, (item, productInfo) -> {
            item.setProductName(productInfo.getName());
            item.setProductSpec(productInfo.getSpec());
            item.setProductUnit(productInfo.getUnit());
        });
        result.setContractDetailsList(salesContractDetailsVos);
        //赋值客户信息
        Customer customer = customerService.getById(result.getCustomerId());
        if(ObjectUtil.isNotEmpty(customer)){
            result.setCustomerName(customer.getName());
            result.setCustomerBank(customer.getBeneficiaryBank());
            result.setCustomerSwiftCode(customer.getSwiftCode());
            result.setCustomerAccountNumber(customer.getBeneficiaryAccountNumber());

            //赋值客户国省市名称
            Map<Long, String> areaMapByIds = CustomizeAreaUtil.getAreaMapByIds(Arrays.asList(
                    customer.getCountryId(),
                    customer.getProvinceId(),
                    customer.getCityId())
            );
            result.setBuyCountryName(areaMapByIds.get(customer.getCountryId()));
            result.setBuyProvinceName(areaMapByIds.get(customer.getProvinceId()));
            result.setBuyCityName(areaMapByIds.get(customer.getCityId()));
            result.setBuyAddress(customer.getAddress());
            result.setBuyDutyParagraph(customer.getDutyParagraph());
        }
        //赋值客户更多联系人
        List<CustomerUser> customerUserList = customerUserService.list(q -> q.eq(CustomerUser::getCustomerId, salesContract.getCustomerId()));
        result.setCustomerUserList(customerUserList);

        //赋值卖方信息多公司
        AccountManagement accountManagement = accountManagementService.getById(result.getShroffAccountId());
        Corporation corporation = corporationService.getById(result.getSellCorporationId());
        result.setSellCorporationName(corporation.getName());
        result.setSellCorporationNumber(corporation.getCorporationNumber());
        result.setSellTaxationCode(corporation.getTaxationCode());

        result.setSellAccountName(accountManagement.getName());
        result.setSellOpeningBank(accountManagement.getOpeningBank());
        result.setSellInterbankNumber(accountManagement.getInterbankNumber());
        result.setSellAccountOpening(accountManagement.getAccountOpening());

        return result;
    }

    @DSTransactional
    @Override
    public void add(SalesContractDto salesContractDto) {
        List<SalesContractDetails> salesContractDetailsList = salesContractDto.getSalesContractDetailsList();
//        BigDecimal count = BigDecimal.ZERO;
//        for (SalesContractDetails salesContractDetails : salesContractDetailsList) {
//            count = count.add(salesContractDetails.getUnitPrice().multiply(salesContractDetails.getQuantity()));
//        }
//        salesContractDto.setContractAmount(count);
        salesContractDto.setCode(CodeEnum.SALES_CONTRACT.getCode());
        this.save(salesContractDto);
        for (SalesContractDetails salesContractDetails : salesContractDetailsList) {
            salesContractDetails.setSalesContractId(salesContractDto.getId());
        }
        salesContractDetailsService.saveBatch(salesContractDetailsList);
    }

}
